/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/*          EXIFPROBE - TIFF/JPEG/EXIF image file probe               */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Copyright (C) 2002 by Duane H. Hesser. All rights reserved.        */
/*                                                                    */
/* See the file LICENSE.EXIFPROBE for terms of use.                   */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#ifndef lint
static char *ModuleId = "@(#) $Id: maker_casio.c,v 1.10 2003/07/01 00:43:29 alex Exp $";
#endif

/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Casio camera maker-specific routines.                              */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */
/* Most of the information coded here is due to Eckhard Henkel        */
/* (eckhard.henkel@t-online.de) as shown by TsuruZoh Tachibanaya at:  */
/* http://www.ba.wakwak.com/~tsuruzoh/Computer/Digicams/exif-e.html   */
/* =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=- */

#include <stdio.h>
#include <string.h>
#include "defs.h"
#include "extern.h"
#include "misc.h"
#include "maker.h"

/* Find the identifying number assigned to known Casio camera models. */
/* This number is used to dispatch print and interpret routines       */
/* approopriate to the current image.                                 */

int
casio_model_number(char *model,char *software)
{
    int number = GENERIC;

    if(model)
    {
        switch(*model)
        {
            case 'Q':
                if(strcmp(model,"QV-2000UX") == 0)
                    number = CASIO_QV2000UX;
                else if(strcmp(model,"QV-3000EX") == 0)
                    number = CASIO_QV3000EX;
                else if(strcmp(model,"QV-8000SX") == 0)
                    number = CASIO_QV8000SX;
                break;
            default:
                number = GENERIC;
                break;
        }
    }
    return(number);
}

/* Dispatch a Casio print routine based upon model                    */

void
print_casio_makervalue(struct ifd_entry *entry_ptr,int make, int model,
                    char *prefix)
{

    if(entry_ptr && (PRINT_VALUE))
    {
        switch(model)
        {
            case CASIO_QV2000UX:
            case CASIO_QV3000EX:
            case CASIO_QV8000SX:
            case GENERIC:
            default:
                print_casio1_makervalue(entry_ptr,make,model,prefix);
                casio1_interpret_value(entry_ptr);
                break;
        }
    }
}

/* Model-specific print routine for Casio cameras. This routine is    */
/* responsible for picking off any direct entry tags which are        */
/* peculiar and will not be handled properly by print_value()         */
/* (usually UNDEFINED values which fit in the 4-byte entry value). If */
/* there are no such entries, this function simply calls              */
/* print_value().                                                     */

void
print_casio1_makervalue(struct ifd_entry *entry_ptr,int make, int model,
                    char *prefix)
{
    if(entry_ptr && (PRINT_VALUE))
    {
        switch(entry_ptr->tag) 
        {
            default:
                print_value(entry_ptr,PREFIX);
                break;
        }
    }
}


/* Dispatch a routine to decode and print offset values for Casio     */
/* cameras.                                                           */

void
print_casio_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{
    if(entry_ptr && (PRINT_VALUE))
    {
        switch(model)
        {
            default:
                casio1_offset_makervalue(inptr,byteorder,entry_ptr,
                                            offset_ref,prefix,indent,
                                            addrwidth,make,model,at_offset);
                break;
        }
    }
}

/* Model-specific routine to print UNDEFINED values found at offsets  */
/* in Casio makernotes.                                               */

void
casio1_offset_makervalue(FILE *inptr,unsigned short byteorder,
    struct ifd_entry *entry_ptr,unsigned long offset_ref,char *prefix,
    int indent,int addrwidth,int make,int model,int at_offset)
{
    if(entry_ptr)
    {
        switch(entry_ptr->tag)
        {
            default:
                print_generic_offset_makervalue(inptr,byteorder,entry_ptr,
                                        offset_ref,prefix,indent,
                                        addrwidth,make,model,at_offset);
                break;
        }
    }
}


/* Casio-specific tagnames for makernotes.                            */

char *
maker_casio_tagname(unsigned short tag,int model)
{
    char *tagname = (char *)0;

    /* Check specific models first                                    */
    switch(model)
    {
        /* Add special-case models here                               */
        default:
            break;
    }

    /* If no model-specific tag is found, use a "generic" Casio tag   */
    if(tagname == NULL)
    {
        switch(tag)
        {
            case 0x0001: tagname = "RecordingMode"; break;
            case 0x0002: tagname = "Quality"; break;
            case 0x0003: tagname = "FocusingMode"; break;
            case 0x0004: tagname = "FlashMode"; break;
            case 0x0005: tagname = "FlashIntensity"; break;
            case 0x0006: tagname = "ObjectDistance"; break;
            case 0x0007: tagname = "WhiteBalance"; break;
            case 0x000a: tagname = "DigitalZoom"; break;
            case 0x000b: tagname = "Sharpness"; break;
            case 0x000c: tagname = "Contrast"; break;
            case 0x000d: tagname = "Saturation"; break;
            case 0x0014: tagname = "ISO_CCDSensitivity"; break;
            default: break;
        }
    }
    return(tagname);
}


/* Report the "meaning" of tag values found directly in a Casio       */
/* MakerNote IFD entry (not at an offset).                            */

void
casio1_interpret_value(struct ifd_entry *entry_ptr)
{
    switch(entry_ptr->tag)
    {
        case 0x0001:    /* RecordingMode */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Single Shutter"); break;
                case 2: printf("Panorama"); break;
                case 3: printf("Night Scene"); break;
                case 4: printf("Portrait"); break;
                case 5: printf("Landscape"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x0002:    /* Quality */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Economy"); break;
                case 2: printf("Normal"); break;
                case 3: printf("Fine"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x0003:    /* Focusing Mode */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 2: printf("Macro"); break;
                case 3: printf("Auto"); break;
                case 4: printf("Manual"); break;
                case 5: printf("Infinity"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x0004:    /* Flash Mode */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Auto"); break;
                case 2: printf("On"); break;
                case 3: printf("Off"); break;
                case 4: printf("Redeye"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x0005:    /* Flash Intensity */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 11: printf("Weak"); break;
                case 13: printf("Normal"); break;
                case 15: printf("Strong"); break;
                default: printred("undefined"); }
            break;
        case 0x0006:    /* Object Distance */
            pushcolor(UNCERTAIN_COLOR);
            printf(" mm");
            popcolor();
            break;
        case 0x0007:    /* White Balance */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 1: printf("Auto"); break;
                case 2: printf("Tungsten"); break;
                case 3: printf("Daylight"); break;
                case 4: printf("Fluorescent"); break;
                case 5: printf("Shade"); break;
                case 129: printf("Manual"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x000a:    /* Digital Zoom */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0x10000: printf("Off"); break;
                case 0x10001: printf("On"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x000b:    /* Sharpness */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Normal"); break;
                case 1: printf("Soft"); break;
                case 2: printf("Hard"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x000c:    /* Contrast */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Normal"); break;
                case 1: printf("Low"); break;
                case 2: printf("High"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        case 0x000d:    /* Saturation */
            print_startvalue();
            switch(entry_ptr->value)
            {
                case 0: printf("Normal"); break;
                case 1: printf("Low"); break;
                case 2: printf("High"); break;
                default: printred("undefined"); break;
            }
            print_endvalue();
            break;
        default:
            break;
    }
}
